# (C) Copyright 1995 - 2021 Micro Focus or one of its affiliates.

# The only warranties for products and services of Micro Focus and its affiliates and licensors
# ("Micro Focus") are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions contained
# herein. The information contained herein is subject to change without notice.

# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items  are licensed
# to the U.S. Government under vendor's standard commercial license.


# Copyright (C) [2006-2007] Novell, Inc.  All Rights Reserved.

# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
# DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN
# CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
# SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.

# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
# AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
# OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE,
# ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER
# DEALINGS IN THE WORK.

require 'fiddle'
require 'fiddle/function'

module Migration
  module KeyStore


    $:.unshift File.expand_path(File.join(File.dirname(__FILE__), '..'))
    require 'migration/yaml'
    $:.shift

    unless defined? MIGPASSWD
      MIGPASSWD = "#{ENV['HOME']}/.migration"
    end
    unless defined? MIGKEY
      MIGKEY = "Novell.Migration"
    end

    # credentials is used as a catch all for asking for and retrieving
    # credentials that are required. 'store' will specify that if credentials
    # are to be stored/retrieved to/from OCS or not
    # Delete will specify that if credentials are retrieved from the
    #credential store that they will be ignored.
    def KeyStore.credential(key, type, useCASA = false, delete = false)
      puts "KeyStore:credential: called" if $MIG_DEBUG

      if not File.exist? '/opt/novell/migration/sbin/libocsimpl.so'
        raise MigrationException.new("OES Credential Store library not found on the system\n")
      end
      if not File.exist? '/usr/sbin/oescredstore'
        raise MigrationException.new("OES Credential Store is not found on the system\n")
      end
      options = nil
      storeRetVal = nil

      begin
        options = self.retrieve(:basic, key, type, useCASA)
      rescue MigrationException
        raise
      rescue StandardError
        # Key was not in CASA or File,  get the key
      end

      if delete and not options.nil? and not options.empty?
        options.clear
      end

      if options.nil? or options.empty?
        username, password = self.prompt(key, type)

        options = {}
        options[:password] = password

        self.convert_username(username, type).each_pair do |k,v|
           options[:type] = k
           options[:username] = v
           options[:password] = password
           storeRetVal = self.store(:basic, key, options, useCASA)
         end

        options[:username] = username
        options[:password] = password
        if useCASA and storeRetVal > -1 and not storeRetVal.nil?
            puts "Successfully stored user credentials in OES Credential Store"
        end
      end

      options.delete(:type)

      return options
    end
        
    # Retrives credentials from the OCS secret store or migraiton file.
    # NOTE:   Binary credentials are not supported in CASA 1.7
    # 
    # Arguments:
    #   type   Credential type :basic or :binary
    #   Key    String that the secret will be stored under, ex. dns1.novell.com
    #   id     For type :basic values are:
    #            :cn       Common Name (CN), ex. John Smith
    #            :nds_dn   Distinguished Name (ND_NDAP), ex. admin.novell
    #            :nds_fdn  Fully Distinguished Name (FDN_NDAP), ex. cn=admin.o=novell
    #            :ldap_dn  Full Distinguished LDAP Name (DN_LDAP), ex. cn=admin,o=novell
    #            :email    Email Address, ex. admin@novell.com
    #            :other    Non specified credential

    #          For type :binary value is a string.
    #
    # Throws: SystemError, ArgumentError, NotImplementedError
    #
    # :call-seq:
    #   KeyStore::retrieve(:basic, key, id) -> { :username, :password }
    #   KeyStore::retrieve(:binary, key, id) -> { :data }
    #

    def KeyStore.retrieve(type, key, id, useCASA)
        puts "KeyStore:retrieve: called" if $MIG_DEBUG

        if useCASA
            options = nil
            libm = Fiddle.dlopen('/opt/novell/migration/sbin/libocsimpl.so')
            username = ""
            password = ""

            retObj = Fiddle::Function.new(
                            libm['load_pwd_from_ocs'],
                            [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP],
                            Fiddle::TYPE_INT
                        )
            server = key + id.to_s

            retVal = retObj.call(server, Fiddle::Pointer.to_ptr(username), Fiddle::Pointer.to_ptr(password))
            if retVal.to_i > 0
                options = {}
                options[:username] = Fiddle::Pointer.to_ptr(username).to_s
                options[:password] = Fiddle::Pointer.to_ptr(password).to_s
            end
            return options
        else
            return self.get( key, id )
        end
    end

    # Sets the credential in the OCS store or migration file.
    # NOTE: Binary credentials are not supported in CASA 1.7
    #
    # Arguments:
    #   type   Credential type :basic or :binary
    #   Key    String that the secret will be stored under, ex. dns1.novell.com
    #
    #   Options Hash:
    #   if type is :basic then the required options are:
    #     :type      Type of credential being stored - :cn, :nds_dn, :nds_fdn, :ldap_dn, :email, :other
    #     :username  Username that matches credential type
    #     :password  Password for username
    #
    #   if type is :binary then the required options are:
    #     :id   The name of the secret id.
    #     :data The data to be stored in CASA
    #
    # Throws: SystemError, ArgumentError, NotImplementedError
    #

    def KeyStore.store(type, key, options, useCASA)
      puts "KeyStore:store: called" if $MIG_DEBUG

        if useCASA
            libn = Fiddle.dlopen('/opt/novell/migration/sbin/libocsimpl.so')

            storeObj = Fiddle::Function.new(
                      libn['save_passwords_to_ocs'],
                      [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP],
                        Fiddle::TYPE_INT
                      )
            server = key + options[:type].to_s
            retVal = storeObj.call(server, options[:username], options[:password])
            if retVal.to_i < 0
                puts "Failed to store user credentials in OES Credential Store"
            end
        else
            self.put(type, key, options)
        end

        options.clear
        return retVal.to_i
    end


    # Removes the credential from the OCS store or migration file.
    # NOTE: Removal of shared secret id's is not supported in CASA 1.7
    #
    # Arguments:
    #   key       The ID or secret of the type of credential being stored.
    #
    def KeyStore.remove( key )
      puts "KeyStore:remove: called" if $MIG_DEBUG
      successCount = 0
      totalCount = 0
      options = nil
      skipped_credtype = Array.new
      credtypes = Array.new
      credtypes = KeyStore.getcredentialtypes()
      libr = Fiddle.dlopen('/opt/novell/migration/sbin/libocsimpl.so')

      removeObj = Fiddle::Function.new(
                    libr['remove_passwords_from_ocs'],
                    [Fiddle::TYPE_VOIDP],
                    Fiddle::TYPE_INT
                    )
      if not key.nil?
        credtypes.each do |cred|
          server = key + cred
          options = self.retrieve(:basic, key, cred, useCASA = true)
          if not options.nil?
            retVal = removeObj.call(server)
            if retVal.to_i == 0
              successCount = successCount.next
            else
              skipped_credtype.push(server)
            end
            totalCount = totalCount.next
          end
          server = ""
        end
      end

      if successCount == totalCount
        puts "Successfully removed user credentials from OES Credential Store"
      else
        puts "Failed to remove the following user credentials from OES Credential Store"
        puts " #{skipped_credtype.join("\n  ")} "
        puts "Try to execute command again or use oescredstore command to delete."
      end
    end



    # Prompts the user for the credentials given by key and type.
    #
    # Arguments:
    #   Key    String that the secret will be stored under, ex. dns1.novell.com
    #   type   Values can be:
    #            :cn       Common Name (CN), ex. John Smith
    #            :nds_dn   Distinguished Name (ND_NDAP), ex. admin.novell
    #            :nds_fdn  Fully Distinguished Name (FDN_NDAP), ex. cn=admin.o=novell
    #            :ldap_dn  Full Distinguished LDAP Name (DN_LDAP), ex. cn=admin,o=novell
    #            :email    Email Address, ex. admin@novell.com
    #            :other    Non specified credential
    #
    # :call-seq:
    #   KeyStore::prompt( key, type) -> username, password
    #   
    def KeyStore.prompt( key, type )
      puts "KeyStore:prompt: called" if $MIG_DEBUG          
      username, password = nil, nil
      count = 0
      
      catch (:done) do 
        example = case type
          when :cn    then   "administrator"
          when :nds_dn then  "admin.mycompany"
          when :nds_fdn then "cn=admin.o=mycompany"
          when :ldap_dn then "cn=admin,o=mycompany"
          when :email  then  "admin@mycompany.com"
          when :other  then  "root"
#          else
#            raise ArgumentError, "Invalid type: #{type}", caller
          end
      
        while true
          $stderr.puts "Enter the username (e.g #{example}) "
          $stderr.print "for the server #{key}: "

          username = $stdin.gets.chomp

          if username.empty? or username.nil?
            $stderr.puts "Username cannot be blank!"

            count += 1
            next if count <= 2

            raise ArgumentError,
                "Invalid username, you must enter a valid user name.", caller
          end

          #Basic validation
          if not type == :cn and not type == :other and not type == :email
            if type == :nds_dn
              unless username.include?(".")
                $stderr.puts "Username must be in the form of admin.mycompany"
              else
                throw :done
              end
            elsif type == :nds_fdn 
              unless username.include?("cn=") or username.include?(".")
                $stderr.puts "Username must be in the form of cn=admin.o=mycompany"             
              else
                throw :done
              end
            elsif type == :ldap_dn 
              unless username.include?(",") and username.include?("cn=")
                $stderr.puts "Username must be in the form of cn=admin,o=mycompany"
              else
                throw :done
              end
            end

            count += 1
            next if count <= 2

            raise ArgumentError,
                "Invalid username, you must enter a valid user name.", caller
          end 

          throw :done
        end
      end
   
      #Used to catch <ctrl>-c interupts.  The terminal would not
      #echo keystrokes back to the console with out it.
      begin
        $stderr.print "Enter #{username} password: "
	if $stdin.isatty
        `stty -echo`
	end
        password = $stdin.gets.chomp
	if $stdin.isatty
        `stty echo 2>/dev/null`
	end
        $stderr.print "\n"    
      ensure
	if $stdin.isatty
        `stty echo`
	end
      end     
      return username, password   
    end

    def KeyStore.get( key, id )
      return nil if $records.nil?
      return nil unless $records.has_key?(key)

      record = $records[key]
      found = false
      options = Hash.new
      record.each { |i|
        if i.has_key?(id)
          if id == :cn or id == :nds_dn or id == :nds_fdn or id == :ldap_dn or
          id == :email or id == :other 
            options [:username] = i[id]
            found = true
          end
        elsif i.has_key?(:password)
          options[:password] = i[:password]
        end
      }

     if found == true
        return options
      else
        return nil
      end
    end

    def KeyStore.put( type, key, options )
      unless $records.class == Hash
        $records = {}
      end

      key_record = $records[key]
      key_record = Array.new if key_record.nil? or key_record.empty?

      if type == :binary
        key_record[options[:id]] = options[:data]
      else
        key_record << { options[:type] => options[:username] }
        key_record<< {:password => options[:password]}
      end

      $records[key] = key_record

      options.clear

      return true
    end

    # Removes key from the MIGPASSWD file
    def KeyStore.remove_file( key )
      puts "KeyStore:remove_file: called" if $MIG_DEBUG            
      raise ArgumentError,  "Cannot remove key from migration password file. Key is empty or nil.", 
        caller if key.empty? or key.nil?

      records = {}
      
      if File.exists?( MIGPASSWD )
        records = YAML.load( File.open( MIGPASSWD ))
      else
        return false
      end
      
      if records.has_key?( key )
        records.delete( key )
        
        File.open( MIGPASSWD, 'w', 0600 ) do |migpasswd| 
          YAML.dump( records , migpasswd)
        end      
      end
      
      records.clear
      
      return true
    end

    ################################################
    # Some helper functions left from migenv
    ################################################
    def KeyStore.convert_username(username, type)
      puts "KeyStore:convert_username: called" if $MIG_DEBUG      	
      usernames = {}

      return { :cn => username } if type == :cn
      return { :other => username } if type == :other
      return { :email => username } if type == :email

      if type == :ldap_dn
        usernames[:ldap_dn] = username
        usernames[:cn] = username[username.index("=")+1..username.index(",")-1]
        usernames[:nds_dn] = (username.split(",").map! { |x| x[x.index("=")+1..-1]}).join('.')
        usernames[:nds_fdn] = username.gsub(',', '.')
      elsif type == :nds_fdn
        usernames[:nds_fdn] = username
        usernames[:cn] = username[username.index("=")+1..username.index(".")-1]
        usernames[:nds_dn] = (username.split(".").map! { |x| x[x.index("=")+1..-1]}).join('.')
        usernames[:ldap_dn] = username.gsub('.', ',')
      elsif type == :nds_dn
        usernames[:nds_dn] = username
        usernames[:cn] = username.split(".").first
      end

        return usernames
    end


    def KeyStore.getcredentialtypes()
      %w(ldap_dn cn nds_dn nds_fdn other email)
    end

  end
end
