#!/usr/bin/env ruby
# Copyright (C) [2006-2007] Novell, Inc.  All Rights Reserved.

# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
# DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN
# CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
# SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.

# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
# AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
# OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE,
# ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER
# DEALINGS IN THE WORK.
require 'optparse'
require 'ostruct'
require 'pp'

ParseArgText = :ParseArgText
ParseArgBool = :ParseArgBool
ParseArgInt = :ParseArgInt
ParseArgFloat = :ParseArgFloat
ParseArgMult = :ParseArgMult
ParseArgOrd = :ParseArgOrd     # an ordered positional arg, without - or --

ParseArgHelp = ['h', 'help', "show this help message", nil]
ParseArgUsage = [nil, 'usage', "display brief usage message", nil]

def parsetag(parsearg, join=", ", append=nil)
	# Combines the short and long opts together in a help string.
	# join is what to put between short and long opt.
	# append is what to put at the end of the short and long opt.  If there
	# is only a short opt, we will append instead a number of spaces equal
	# to append. (this feature is for --usage, e.g., so that we can get
	# '-i VAL' or '-i|--int=VAL' without further magic)
	tag = "" 
	if parsearg[0]
		tag = "-#{parsearg[0]}"
		appendConnect = " "
	end
	if parsearg[1]
		if tag != ""
			tag += join 
		end
		tag +="--#{parsearg[1]}"
		appendConnect = "="
	end
	if append
		if parsearg[3] == ParseArgText
			tag += appendConnect+"TEXT"
		elsif parsearg[3] == ParseArgInt
			tag += appendConnect+"INT"
		elsif parsearg[3] == ParseArgFloat
			tag += appendConnect+"FLOAT"
		elsif parsearg[3] == ParseArgMult
			tag += appendConnect+"VAL1[ VAL2[ VAL3 ...]]"
		end
	end
	if parsearg[3] == ParseArgOrd
		tag = parsearg[1]
	end
	return tag
end
	
def printHelp(argdef)
	puts "Usage: " + `basename #{$0}`.chomp! + " [OPTION...]\n"
	for j in argdef
		tag = parsetag(j)
		if tag.length < 33
			tag = tag+" "*(33-tag.length)
		end
		desc = j[2]
		if desc.length+40 > 80
			# formatting for 80-col display
			def cutstring(s, width=40)
				if s.length <= width
					return [s]
				end
				i = s.rindex(" ",width)
				f = s[0,i]
				r = cutstring(s[i..-1],width)
				r.insert(0,f)
				return r
			end
			lines = cutstring(desc)
			pad = " "*(43-1)
			desc = lines.join("\n"+pad)
		end
		puts "  #{tag}        #{desc}"
	end
end

class ParseArgs
	def ParseArgs.printHelp(argdef)
		printHelp(argdef)
	end
end

def parseargs(args, argdef)
	begin

	def printUsage(argdef)
		s = "Usage: " + `basename #{$0}`.chomp! + " "
		for j in argdef
			tag = parsetag(j, "|", true) 
			if not j[4]
				s += "["+tag+"] "
			else
				s += tag+" "
			end
		end
		s.gsub!(/=TEXT/," TEXT")
		$stderr.puts s
	end


	
	# help and usage messages
	if argdef.detect { |d| d == ParseArgHelp } and 
			args.detect { |a| a == '-h' or a == '--help' }
		printHelp(argdef)
		exit 0
	elsif argdef.detect { |d| d == ParseArgUsage } and
			args.detect { |a| a == '--usage' }
		printUsage(argdef)
		exit 0
	end

	
	# save a deep copy of argdefOrig for error messages
	argdefOrig = []
	for i in argdef
		argdefOrig << i.clone()
	end

	# make dicts out of argdef
	argdefDict = {}
	argdefLongDict = {}
	for i in argdefOrig
		if i[0] == nil
			i[0] = i[1]
		end
		argdefDict[i[0]] = i
		argdefLongDict[i[1]] = i[0]
	end
	
	options = {}
	backargs = []
	
	# pre-process args 
	count = 0
	for i in args
		if i[0..1] == '--' and i.index('=')
			# turn any longopts like "--[opt]=[val]" into "--[opt] [val]"
			lopt = i[0..i.index('=')-1]
			lval = i[i.index('=')+1..-1]
			args[count] = lval
			args.insert(count,lopt)
		elsif i[0,1] == '-' and i[0..1] != '--' and i.length > 2
			# expand all short opts of the form "-abc" into "-a -bc" (iterative
			# -- this will result in "-a -b -c")
			args[count] = "-#{i[2..-1]}"
			args.insert(count, "-#{i[1,1]}")
		end
		count += 1
	end
	# process args
	opt = nil
	for i in args
		#puts "examining #{i}"
		if i[0..1] == '--'
			opt = argdefDict[argdefLongDict[i[2..-1]]]
			#puts "  longopt"
			if opt == nil
				errorString = "Illegal option #{i}"
				raise MigSyntaxError.new(errorString)
			end
			# We need to set the options[opt[0]] to nil for checking whether the particular key
                        # is present in 'options' hash or not.
                        # Condition is required because in case if we provide a option multiple times,
                        # options[opt[0]] should not be set to nil for next validation check
                        if not options[opt[0]]
                                options[opt[0]] = nil
                        elsif opt[3] != ParseArgMult
                                errorString = "Option '#{opt[0]}' cannot be specified more than once"
                                raise MigSyntaxError.new(errorString)
                        end
		elsif i[0..0] == '-'
			opt = argdefDict[i[1..-1]]
			#puts "  shortopt"
			if opt == nil
				errorString = "Illegal option #{i}"
				raise MigSyntaxError.new(errorString)
			end
                        if not options[opt[0]]
                                options[opt[0]] = nil
                        elsif opt[3] != ParseArgMult
                                errorString = "Option '#{opt[0]}' cannot be specified more than once"
                                raise MigSyntaxError.new(errorString)
                        end
		else
			#puts "  optvalue"
			if opt and opt[3] != ParseArgBool
				# this is the opt from last iteration, assign it the value.
				if opt[3] == ParseArgMult
					#puts "    multoptvalue"
					if options[opt[0]]
						options[opt[0]] << i
					else
						options[opt[0]] = [i]
					end
				else
					#puts "    value"
					if opt[3] == ParseArgInt
						i = i.to_i
					elsif opt[3] == ParseArgFloat
						i = i.to_f
					end
					options[opt[0]] = i
					opt = nil
				end
			else
				#puts "  arg = #{i}"
				backargs[backargs.length] = i
				opt = nil
			end
		end
		if opt and opt[3] == ParseArgBool
			# boolean options don't depend on another arg, so we can set here
			#puts "    boolvalue"
			options[opt[0]] = true
			opt = nil
		end
	end

	# assign positional args
	for i in argdef
		if i[3] == ParseArgOrd and backargs.length > 0
			options[i[1]] = backargs[0]
			backargs.delete(backargs[0])
		end
	end

	 # Following 'for' loop validates whether value is passed along with option or not
        for i in argdefOrig
		# If short opt is null, then change i[0] to i[1] (long opt value)
		if i[0] == nil
			i[0] = i[1]
		end
                if options.has_key?(i[0])
                        if not options[i[0]] and (i[3] == ParseArgText or i[3] == ParseArgMult)
                                errorString = "Value for option '#{i[0]}' is missing"
                                raise MigSyntaxError.new(errorString)
                        end
                        if (options[i[0]].to_s.strip == "") and (i[3] == ParseArgText or i[3] == ParseArgMult)
                                errorString = "Value for option '#{i[0]}' is missing"
                                raise MigSyntaxError.new(errorString)
                        end
                end
        end

	# check for missing required args
	for i in argdefOrig
		if i[4] and ((i[0] and not options[i[0]]) or 
				(i[3] == ParseArgOrd and not options[i[1]]))
			missing = parsetag(i, "|", true)
			errorString = "option #{missing} must be specified"
                        raise MigSyntaxError.new(errorString)
			#raise SystemExit, -1
		end
	end

	return [options, backargs]
	rescue
		printUsage argdef
		raise
	end
end


def main 
	# Here's an example of how to use parseargs, showing all accepted types.
	# The format is: [shortopt, longopt, description, type, required].
	# The return value is a pair, the first member is a dictionary, with the
	# short option name as the key for any values given in args, and the second
	# a list of arguments that didn't belong to any option.
	# You can use 'nil' in any of the positional parameters you'd like ignored
	# (if you use 'nil' for a short opt, the long opt becomes the keyword in
	# the options dict returned).
	# Try invoking this with, for example: 
	# '-h', 
	# '--help', 
	# '--usage'
	# '-m monkey cheese it -c MySSNfile -l hello -n Joe -I 4 -f 4.3 there --hate4=10', 
	# '-n Henry -I 12 --hate3 12 -lc mySSNFile'
	# etc.
	argdef = [
		['l', 'lucky', "Are you feely lucky?  Well, are you, punk?", 
			ParseArgBool],
		['m', 'mimes', "List all the mimes you want sacked", ParseArgMult],
		['n', 'name', "Your childhood nemesis", ParseArgText, true],
		['i', 'num', "Your most hated whole number", ParseArgInt],
		['I', nil, "Your 2nd most hated whole number", ParseArgInt, true],
		[nil, 'hate3', "Your best friend's most hated number", ParseArgInt,
			true],
		[nil, 'hate4', "Your 2nd best friend's most hated number -- not your best friend's 2nd most hated number.  That's an important distinction", ParseArgInt],
		['f', 'float', "Your age, expressed as a decimal number", 
			ParseArgFloat],
		['c', 'crazy', "Are you clinically insane?", ParseArgBool],
		[nil, 'filename', "A filename that contains SSN and Credit Card", 
			ParseArgOrd, true],
		ParseArgHelp,   # optional
		ParseArgUsage   # optional
	]

	options, args = parseargs(ARGV, argdef)

	puts "\nHere's what we got back from parseargs:"
	puts "--options--"
	pp options
	puts "--args--"
	pp args
end

main if __FILE__ == $0
