# Copyright (C) [2006-2007] Novell, Inc.  All Rights Reserved.

# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
# DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN
# CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
# SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.

# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
# AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
# OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE,
# ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER
# DEALINGS IN THE WORK.

# :title: Novell Migration Utility - ntuserls
# ==Description: 
# Class for processing NTFS rights
#  
module Migration
  module Windows
    module Rights
      $:.unshift File.expand_path(File.join(File.dirname(__FILE__)))
      require 'ntfs_values' 
      $:.shift         

      $:.unshift File.expand_path(File.join(File.dirname(__FILE__), '..'))
      require 'exceptions'
      $:.shift         
          
      class NTFS_rights
        include Migration::Windows::Rights::NTFS
        
        attr_reader :raw_type, :raw_flags, :raw_rights
        
        def initialize( rights = nil )
          if rights.nil?
            @raw_type = 0
            @raw_flags = 0
            @raw_rights = 0x0
          else
            self.from_s( rights )
          end
          
          @ntfs_type = NTFS_Type.new
          @ntfs_flags = NTFS_Flags.new
          @ntfs_rights = NTFS_Masks.new
        end
        
        def has_inheritance?
          @raw_flags == 0 ? false : true 
        end 
               
        # Takes a rights string in the format of nn/nn/0xhhhhhhhh
        # 
        # :call-seq:
        #   NTFS.get_rights( raw_rights = "" ) -> string, array, array
        #        
        def from_s( rights )
          #Rights string from smbcacls
          if rights =~ /[0-9]*\/[0-9]*\/0x[0-9a-fA-F].*/                  
            rights_array = rights.split('/')
            @raw_type = rights_array[0].to_i
            @raw_flags = rights_array[1].to_i
            @raw_rights = rights_array[2].hex
          else
            raise RightsError, "Cannont translate #{rights} into valid NTFS rights", caller
          end
        end
        
        def from_y( rights )
          if rights.class == Array
            rights.each { |right|
              self.from_right( right )
            }
          elsif rights.class == String  
            self.from_right( rights )
          else
            raise RightsError, "Cannont translate #{rights.inspect} into valid NTFS rights"
          end
        end
        
        def to_s( )
          return self.get_type + ", "
                 self.get_flags.join(", ") + ", "
                 self.get_rights.join(", ")
        end
        
        def to_ary()
          return [ self.get_type,
                   self.get_flags,
                   self.get_rights  ]
        end
        
        # Parse NTFS Type
        # 
        # :call-seq:
        #   NTFS.get_type -> string
        #   
        def get_type
          return @ntfs_type.index(@raw_type)
        end
        
        # Parse flags into an array with string
        # representations of the actual values
        # 
        # :call-seq:
        #   get_flags -> array
        #            
        def get_flags
          flags = []
#          mask = 0x1
#          
          return flags if @raw_flags == 0x0

          if @ntfs_flags.has_value?(@raw_flags)
            flags << @ntfs_flags.index(@raw_flags)
          else
                      
            raw = @raw_flags
            val = MAX_NTFS_FLAGS
            
            8.times do 
              if raw / val > 0
                flags << @ntfs_flags.index( val )
              end
              raw = raw % val
              val /= 2
            end            
          end 
          return flags
        end      
        
        # Parses the rights mask
        # 
        # :call-seq:
        #   get_rights -> array
        #        
        def get_rights
          rights = []
          
          return rights if @raw_rights == 0x0
          
          if @ntfs_rights.has_value?( @raw_rights )
            rights << @ntfs_rights.index( @raw_rights ) 
          else
            raw = @raw_rights
            val = MAX_NTFS_MASKS
            
            32.times do 
              if raw / val > 0
                rights << @ntfs_rights.index( val )
              end
              raw = raw % val
              val /= 2
            end
          end
          
          return rights
        end     

        def from_right( right )
          case right
          when /^NTFS_TYPE_*/
            self.from_type( right )
          when /^NTFS_FLAG_*/
            self.from_flags( right )
          else 
            self.from_rights( right )
          end         
        end
      
        def from_type( type = "" )
          @raw_type |= NTFS_Type.new[type]
        end

        def from_flags( flags = [] )
          return if flags.empty?
          flags.each do |flag|
            @raw_flags |= @ntfs_flags[flag]
          end     
        end

        def from_rights( rights = [] )
          return if rights.empty?
          
          rights.each do |right|
            if @ntfs_rights.has_key? "#{right}"
              @raw_rights |= @ntfs_rights.lcr_value( right )
            else
              raise RightsError, 
                "Invalid right #{right} for NTFS rights.", 
                 caller
            end 
          end                               
        end
        
        def +( other )
          if other.class === self 
            @raw_type |= other.raw_type
            @raw_flags |= other.raw_flags            
            @raw_rights |= other.raw_rights            
          else
            raise RightsError, 
              "Cannot add #{other.class} to NTFS rights. Invalid + operation", caller
          end
        end  
        
        # TODO: - Needs to be added for code complete
        def -( other )
            raise RightsError, 
              "Cannot remove #{other.class} from NSS rights. Operation not implemented", caller
        end        
             
        def method_missing(method_id, *args)
          method_name = method_id.id2name

          if method_name =~ /^has_ntfs_flag.*\?$/
            self.get_flags.include? method_missing_has( method_name )
          elsif method_name =~ /^has_ntfs_type.*\?$/
            self.get_type.eql? method_missing_has( method_name )
          elsif method_name =~ /^has_.*\?$/          
            self.contains?( @raw_rights, @ntfs_rights.lcr_value( method_missing_has( method_name )))
          elsif method_name =~ /^add_ntfs_flag/
            @raw_flags |= @ntfs_flags[ method_missing_add( method_name )]
          elsif method_name =~ /^add_ntfs_type/             
            @raw_type = @ntfs_type[ method_missing_add( method_name )]
          elsif method_name =~ /^add_/
            @raw_rights |= @ntfs_rights.lcr_value( method_missing_add( method_name ))
          elsif method_name =~ /^remove_ntfs_flag/
            value = @ntfs_flags[ self.method_missing_remove( method_name )]
            @raw_flags ^= ( self.contains?( @raw_flags, value ) ? value : 0x0 )
          elsif method_name =~ /^remove_ntfs_type/             
            value = @ntfs_type[ self.method_missing_remove( method_name )]
            @raw_type ^= ( self.contains?( @raw_type, value ) ? value : 0x0 )
          elsif method_name =~ /^remove_/
            value = @ntfs_rights.lcr_value( method_missing_remove( method_name ))
            @raw_rights ^= ( self.contains?( @raw_rights, value ) ? value : 0x0 )
          else
            raise RightsError,
              "Method #{method_name} not implemented for NTFS rights", caller
          end
        end

        def contains?( rights, ntfs_const ) 
          if ((rights & ntfs_const) == ntfs_const) then true
          else false
          end        
        end
        
        def method_missing_has( ntfs_const )
          ntfs_const.gsub(/\?$/, '').sub(/^has_/, '').upcase
        end
        def method_missing_add( check )
          check.sub(/^add_/, '').upcase
        end  
        def method_missing_remove( ntfs_const )
          ntfs_const.sub(/^remove_/, '').upcase
        end              
      end
    end
  end
end
