# Copyright (C) [2006-2007] Novell, Inc.  All Rights Reserved.

# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
# DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN
# CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
# SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.

# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
# AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
# OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE,
# ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER
# DEALINGS IN THE WORK.

# :title: Novell Migration Utility - ntuserls
# ==Description: 
# Module methods for using Samba's rpcclient program.  
# 
module Migration
  module Windows
    module SambaLib
      $:.unshift File.expand_path(File.join(File.dirname(__FILE__), '..', 'domain'))
      require 'user_object'
      require 'group_object'
      $:.shift           
          
      # Tests rpcclient to see if username and password are ok returns
      # _true_ for success. 
      # 
      # :call-seq: 
      #   test_rpclient -> bool
      #
      def test_rpcclient
        self.command( :rpcclient, "quit" )       
        return true
      end        
      
      # Gets share absolute path information
      # 
      # :call-seq: 
      #   get_share_info -> string, string
      #   
      # Returns the Drive letter and the path to the share.
      def get_share_info
        self.command( :rpcclient, "netsharegetinfo #{@share}" ) do |line|
          line.strip!

          if line =~ /^path:/
            path = line.split(':')
            path.shift
            path.map! { |x| x.strip }
            return path.first, path.last
          end
        end
        return nil
      end      
   
      # Uses samba rpclient to retrieve SDDL(SID) to User name mappings
      # 
      # :call-seq:
      #   sid_to_username( sid ) -> string
      #   
      # Returns the user name from the domain
      def sid_to_username( sid )
        return @sids[sid] if @sids.has_key? sid
          
        self.command( :rpcclient, "lookupsids #{sid}" ) do |line|
          if line =~ /^S\-\d\-\d\-[\d\-]+\s.+\s\(\d+\)$/ 
            return @sids[sid] = 
              line.scan(/^S[\d\-]+\s(.+)\s\(\d+\)$/).first.to_s.split('\\').reverse.first
          end
        end
        
        return nil
      end
    
      # Enumerates the all the users in the Domain
      # 
      # :call-seq:
      #   get_domain_users -> hash
      #   
      # Returns a hash of user names and rids.  Key = RID, Value = Username
      def get_domain_users
        user_list = Hash.new

        self.command( :rpcclient, "enumdomusers" ) do |line|
          if line =~ /^user:\[.*\]\srid:\[.*\]$/ 
            user = line.match(/^user:\[(.*)\]\srid:\[(.*)\]$/).captures
            user_list[user.last] = user.first
          end
        end
                
        return user_list
      end

      # Enumerates the all the groupss in the Domain
      # 
      # :call-seq:
      #   rpclient_get_domain_groups( source, username, password ) -> hash
      #   
      # Returns a hash of group names and rids.  Key = RID, Value = Group name
      def get_domain_groups
        group_list = Hash.new
        
        self.command( :rpcclient, "enumdomgroups" ) do |line|
          if line =~ /^group:\[.*\]\srid:\[.*\]$/ 
            group = 
              line.match(/^group:\[(.*)\]\srid:\[(.*)\]$/).captures
            group_list[group.last] = group.first
          end
        end

        return group_list
      end
                  
      # Get Domain user specific information with the user name
      # 
      # :call-seq:
      #   get_domain_user( name ) -> User_object
      #   
      # Returns a _User_object_
      # 
      # NOTE: Cannot use rid to retrieve information.  This is a bug in Samba.
      def get_domain_user( name )
        name = name.gsub(/\s/, '\\ ')
        windows_user = Migration::Windows::Domain::WindowsUser.new
        
        self.command( :rpcclient, "queryuser #{name}" ) do |line|
          case line
          when /User\sName\s*:\s+/
            windows_user.user_name = line.split(':').last.strip
          when /Full\sName\s*:\s+/
            windows_user.full_name = line.split(':').last.strip              
          when /Home\sDrive\s*:\s+/
            windows_user.home_drive = line.split(':').last.strip                            
          when /Dir\sDrive\s*:\s+/
            windows_user.dir_drive = line.split(':').last.strip              
          when /Profile\sPath\s*:\s+/
            windows_user.profile_path = line.split(':').last.strip              
          when /Logon\sScript\s*:\s+/
            windows_user.logon_script = line.split(':').last.strip              
          when /Description\s*:\s+/
            windows_user.description = line.split(':').last.strip                           
          when /user_rid\s*:\s+/
            windows_user.user_rid = line.split(':').last.strip                            
          when /group_rid\s*:\s+/
            windows_user.group_rid = line.split(':').last.strip                                          
          end
        end
        
        return windows_user
      end   
      
      # Get Domain group specific information
      # 
      # :call-seq:
      #   get_domain_group( rid ) -> Group_object
      #   
      # Returns a _Group_object_
      # 
      # NOTE: Cannot use group name to retrieve information.  This is a bug in Samba.
      def get_domain_group( rid )
        windows_group = Migration::Windows::Domain::WindowsGroup.new
        windows_group.rid = rid

        self.command( :rpcclient, "querygroup #{rid}" ) do |line|
          case line
          when /Group\sName\s*:\s+/
            windows_group.group_name = line.split(':').last.strip
          when /Description\s*:\s+/
            windows_group.description = line.split(':').last.strip                           
          when /Group\sAttribute\s*:/
            windows_group.attribute = line.split(':').last.strip                          
          end
        end
        
        windows_group.members = self.get_group_members( rid )
        
        return windows_group
      end            
    
      # Get Group members
      # 
      # :call-seq:
      #   get_group_members( rid ) -> Array
      # 
      def get_group_members( rid )         
        rids = Array.new
        
        self.command( :rpcclient, "querygroupmem #{rid}" ) do |line|
          if line =~ /^rid:\[.*\]\sattr:\[.*\]$/ 
            rid = self.check_rid(line.match(/^rid:\[(.*)\]\sattr:\[.*\]$/).captures.first)

            unless rid == INVALID_DOMAIN_OBJECT
              rids << rid
            end
          end
        end
        
        return rids
      end    

      # Translates information from the srvinfo rpc command into a readable hash
      def get_server_info
        server = Hash.new( false )  
        self.command( :rpcclient, "srvinfo") do |line|
          server[:workstation] = true if line.include?('Wk')
          server[:server] = true if line.include?('Sv')
          server[:PDC] = true if line.include?('PDC')        
          server[:DC] = true if line.include?('BDC')        
          server[:time] = true if line.include?('Tim')        
          server[:NT] = true if line.include?('NT')        
          server[:LMB] = true if line.include?('LMB')        
  
          if line =~ /os\s version\s*:\s(.*)/
             server[:version] = "Windows 2003 Server" if $1.include?('5.2')
             server[:version] = "Windows 2000 Server" if $1.include?('5.0')           
             server[:version] = "Windows NT 4.0 Server" if $1.include?('4.0')           
          end
        end
        return server
      end

      # Translates informaiton from the querydominfo rpc command in to a readable hash
      def get_domain_info
        domain = Hash.new( false )  
        self.command( :rpcclient, "querydominfo") do |line|
          case line
          when /Domain:\s(.*)/
            domain[:domain] = $1.strip
          when /Server:\s(.*)/           
            unless $1.nil? or $1.empty?
              domain[:server] = $1.strip
            end
          when /Comment:\s(.*)/           
            unless $1.nil? or $1.empty?
              domain[:comment] = $1.strip
            end
          when /Total\sUsers:\s(.*)/           
            domain[:users] = $1.strip
          when /Total\sGroups:\s(.*)/
            domain[:groups] = $1.strip
          when /Total\sAliases:\s(.*)/
            domain[:aliases] = $1.strip
          when /Server\sRole:\s(.*)/
            domain[:role] = $1.strip
          end
        end
        return domain
      end
    end
  end
end
